<?php

/*
  --------------------------------------------------------------------------
  This program computes a calendrical Julian Day Number table for any given
  year from 4714 BC to 9999 AD.    It covers both the old Julian and modern
  Gregorian calendars.

  SPECIAL NOTES:

  Calendar year convention applied:  Neg|Pos = BC|AD

  Julian Calendar     -4713 Jan 01 Mon  to  9999 Dec 31 Mon
  Gregorian Calendar  -4714 Nov 24 Mon  to  9999 Dec 31 Sat

  There is no calendrical year 0 (zero) , by definition, but
  there is a mathematical year 0 (corresponding to 1 BC).

  --------------------------------------------------------------
  RELATIONSHIP BETWEEN CALENDRICAL AND MATHEMATICAL YEAR VALUES:

  In the formal computations, the mathematical year is used for
  internal work.

  Calendar year convention applied:  Neg|Pos = BC|AD

  Let:
  cY = Calendrical year value
  mY = Mathematical year value

  Then:
  For positive (AD) years:  mY = cY;
  For negative (BC) years:  mY = cY + 1;

  In PHP, these conditions can be combined in the
  single expression:       $mY += ($cY < 0)? 1:0;

  Thus, the calendrical year -100 (100 BC) equates to
  (-100 + 1) = -99 = mathematical year, which is the
  actual value used for 100 BC in the computations.


  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------
*/



/* -------------------------------------
   Initialize output buffer this must be
   the very first command of the program
   when using cookies.
*/
   ob_start();

   $TextArea1Text = '';   $TextArea1Rows = 1;
   $ErrorString = '';

// Set calendar year range limits.
   $YMin = '-4714';
   $YMax = '+9999';

   $TextSize = '22px';
   $TextCols = '43';

// Compute JD number for 1st day of current month.
   $year  = gmdate('Y');
   $month = gmdate('n');
   $JDNum0101 = GregorianToJD($month, 1, $year);

// Set text area text and background colors.
// These will change to white/red on error.
   $TxColor = 'black';
   $BgColor = 'white';

// Set time zone offset to New York City, USA.
   date_default_timezone_set('America/New_York');
   $STZStr = date_default_timezone_get();

// ----------------------------------
// Program version and revision date.
   $_THIS_SCRIPT_NAME_ = BaseName($_SERVER["SCRIPT_FILENAME"]);
   $at  = "&#97;&#116;";   $UTC = "&#85;&#84;&#67;";
   $_REVISION_DATE_ = 'Revised: '.gmdate("l - F d, Y  $at H:i $UTC", FileMTime($_THIS_SCRIPT_NAME_));


// -----------------------------------
// Get current year from system clock.
   $cYear = date("Y");

   $_RUN_ = $_SERVER["SCRIPT_NAME"];

// Define texts for internal HTML title and page header.
   $_INTERNAL_HTML_TITLE_ = "JD Number Table For Any Year";

   $_PAGE_HEADER_TEXT_ = "<b>JD Number Table For Any Year On the Julian or Gregorian Calendar</b><br>
   <span style='font-size:12px;'>Year&nbsp;Range&nbsp;&nbsp;=&nbsp;&nbsp;&minus;4714&nbsp;&nbsp;to&nbsp;&nbsp;9999&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
   &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Neg | Pos&nbsp;&nbsp;=&nbsp; BC | AD&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;J | G = Julian | Gregorian</span>";

/* ----------------------------------------------------
   Define a name for the cookie and set expiration date
   to 30 days if not used or deleted before then.  This
   value is expressed in seconds from the time that the
   cookie is created (Time Now + days*86400 seconds).
*/
   $CookieName = 'JD_Number_Table_Settings';

   $SetToExpireIn30Days = time() + 30*86400;

   $_COOKIE_FOUND_ = isset($_COOKIE[$CookieName]);

   if (!isset($_POST['ComputeButton']))
{

/* If there is a previous cookie, then restore
   the interface settings from it.
*/
   if ($_COOKIE_FOUND_)
  {
   $CookieDataString = $_COOKIE[$CookieName];
   list($Y, $JG) = preg_split("[\|]", $CookieDataString);

   $Y = intval(trim($Y));
  }

else

/* If there is no previous cookie, then set default
   interface values and store them in a new cookie.
*/
  {
   $Y = date('Y');
   $JG = 'G';
   $JGStr = ($JG == 'J')? 'Julian' : 'Gregorian';

   $CookieDataString = "$Y|$JG";

   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
  }
}



   if (isset($_POST['ComputeButton']))
{
   $Y = @trim($_POST['Y']);

   $JG = @trim($_POST['JG']);

   if ($Y == '') {$Y = date('Y');  $JG = 'G';}


   $JG = (strtoupper(substr(trim($JG), 0,1)) == 'J')? 'J':'G';

   $JGStr = ($JG == 'J')? 'Julian' : 'Gregorian';

   $YMin = ($JG == 'J')? -4713 : -4714;

// _________________________
// Tidy up interface values.

   $Y = intval($Y);


// __________________________________
// Save interface settings to cookie.

   $CookieDataString = "$Y|$JG";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);
} // End of  if (isset(_POST[...)



// _____________________
// Check for year error.

   if ($Y < $YMin or $Y == 0 or $Y > $YMax)
     {
       $TxColor = 'white';
       $BgColor = '#DD0000';

       $YZeroStr = '';
       if ($Y <= 0)
       {$YZeroStr = " However, there is\nno calendar year 0 (zero), by definition.";}

  $JGStr = ($JGStr == 'Julian')? "old $JGStr calendar." : $JGStr.' calendar. ';

   $ErrorString =
"=== ERROR ================================

Invalid year value:  '$Y'

The year value must be an integer in the
range from $YMin to $YMax when using the
$JGStr  A negative year is
equated to a BC year. $YZeroStr
";
      $TextArea1Text .= "$ErrorString";
      $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
     }


// --------------------------------------------------
// Display program info if [INFO] button was clicked.

   if (isset($_POST['InfoButton']))
      {
       $ErrorString = '-';
       $TextSize = '18px';
       $TextCols = '76';
       $TextArea1Text =
"
This program computes a table of JD (Julian Day) numbers for each day
of any given year from 4714 BC to 9999 AD on the Julian or the modern
Gregorian calendar.

-------------------------------------
Calendar Year Convention Applied Here

BC years are negative and AD years are positive values. The year after
1 BC (&minus;1) is 1 AD (+1), with no calendar year 0 (zero) in between.

In other words, what would normally be calendar year zero is instead
called 1 BC according to our calendar system.

---------------------------
For the Old Julian Calendar

Used until JD Number 2299160 1582-Oct-14-Thu

Range: &minus;4713 Jan 01 Mon  to  9999 Dec 31 Mon

Mathematically, the origin of the old Julian calendar is:
-4713 Jan 01 Mon, where JD Number = 0

---------------------------------
For Our Modern Gregorian Calendar

Official (Catholic) Origin: JD Number 2299161 1582-Oct-15-Fri

Adopted sporadically over the centuries until most countries
today have adopted it as the standard calendar worldwide for
trade and commerce.

Mathematically, the origin of the Gregorian calendar is:
&minus;4714 Nov 24 Mon, where JD Number = 0

------------------------
PROLEPTIC CALENDAR DATES

Proleptic dates are either future dates yet to come or dates that pre-
date the actual calendar system.  For example, the JD number computed
for 1 Jan, 1500, on our Gregorian calendar would be a proleptic date
because that date is some 82 years prior to the existence of our mod-
ern calendar and represents our calendar mathematically pushed back
in time to a theoretical but not actual date.

We may want to know what a particular date on the old Julian calendar
would be on our modern calendar, if it was in use at that time, which
would be a proleptic Gregorian calendar date.

One value of proleptics date is that if we can compute the Julian Date
of an ancient astronomical event, then we can compute what date that
would correspond to on our modern Gregorian calendar if it were in
use back then.

If we knew that a certain ancient king died two months after a certain
total lunar eclipse took place, today, NASA could possibly compute the
Julian Date of the event and then the correspoding date on our calendar
system. Some ancient dates are defined in years of their ruler's reign
and paleoastronomy helps to fill in some of those historic gaps.

---------------------------------------------------------
DISTINCTION BETWEEN THE JULIAN DAY NUMBER AND JULIAN DATE

The Julian Day number is NOT simply the integer value of the Julian
Date, but the relationship between them is still very simple.

The Julian Day Number contains only the calendar date information.

The Julian Date contains both the calendar date and time information.

-----------------------------
JD NUMBER (JULIAN DAY NUMBER)

The Julian Day number, usually called the JD number, is the sequential
serial number of the corresponding calendar date as reckoned from the
mathematical origin of the calendar system, or JD number = 0.  This
value is always an integer, since it is an exact count of calendar
days without regard to any particular time of day, like a count of
the number of calendar days since the beginning of the year.

Given any general Julian Date (JD) value, to obtain the corresponding
Julian Day number:

\$JDNum = floor(\$JD + 0.5); // = Calendar date serial number


The JD number is always an integer value. It only tells us the calendar
date, but does not tell us the time of day.   Mathematically, the JD
number equates to the Julian Date value at 12:00 noon (or mid-point)
of the corresponding calendar date.

The difference between any two JD numbers equates to the exact number
of calendar days between the respective dates, automatically accounting
for any leap years in between.

----------------------------------------------------
To Determine the Day Of The Week From the JD Number:

Each day of the week from Sunday through Saturday can be given an index
value from 0 to 6.

Adding 1 to the JD number and then dividing by 7, the remainder serves
as a Day-Of-the-Week (DoW) index from 0 to 6, telling us the correspon-
ding day of the week on the calendar, from 0 = Sunday to 6 = Saturday.

DoW = (JDNum + 1) mod 7

DoW index = 0 to 6, where: 0=Sun, 1=Mon, 2=Tue, ..., 6=Sat

This formula applies to both the Julian and Gregorian calendars.

In PHP code, it could be written as:
\$DoW = (\$JDNum + 1) % 7;

Or, using the Julian Date:
\$DoW = (floor(\$JD + 0.5) + 1) % 7;


===========================================================================
THE JD (JULIAN DATE)

Not to be confused with the JD Number (Julian Day Number).

The Julian Date is usually simply called the JD value rather than the JD
number or commonly called the JD.  The Julian Date contains both the cal-
endar date and time of day information and it usually has a decimal part,
with the decimal fraction indicating the time of day.

Numerically,  the JD Number equates  to the Julian Date at 12:00 (noon)
on the date and is only an integer value at that moment.  For any other
moment, the JD will have a non-zero decimal part which can be used to
determine the time of day, as demonstrated by the PHP example below.


-----------------------------------------------------------------------
GIVEN ANY GENERAL JD VALUE ON THE GREGORIAN CALENDAR, THE FOLLOWING PHP
CODE DEMONSTRATES HOW TO EXTRACT THE DATE AND TIME OF DAY ELEMENTS FROM
THAT JULIAN DATE.

=== PHP EXAMPLE BEGINS BELOW ========================

&lt;?php

// EXAMPLE for Julian Date:  JD = 2444059.12345678
// =
// 1979 July 04 Wed  at 14:57:46.666

\$JD    = 2444059.12345678;
\$JDNum = floor(\$JD + 0.5);

list(\$m, \$d, \$Y) = preg_split(\"[\\/]\", JDtoGregorian(\$JDNum));

\$MmmStr  = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*(\$m - 1), 3);
\$DoWStr  = substr('SunMonTueWedThuFriSat', 3*((\$JDNum + 1) % 7), 3);
\$dn      = sprintf('%02d', \$d);

\$hours   = 24*(\$JD - floor(\$JD - 0.5) - 0.5);
\$hh      = floor(\$hours);
\$minutes = 60*(\$hours - \$hh);
\$mm      = floor(\$minutes);
\$seconds = 60*(\$minutes - \$mm);
\$ss      = round(\$seconds);

\$hh = sprintf('%02d', \$hh);
\$mm = sprintf('%02d', \$mm);
\$ss = sprintf('%02d', \$ss);

\$DateTimeStr = \"\$Y \$MmmStr \$dn \$DoWStr \$hh:\$mm:\$ss\";

print \"&lt;pre&gt;\$DateTimeStr&lt;/pre&gt;\";

?&gt;


=== PHP EXAMPLE ENDS ABOVE ==========================
=====================================================


";

       $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
      }



/*
   _______________________________________________________
   If no errors detected, then perform computations using
   given arguments and display the results in TEXTAREA1.
*/

   if (trim($ErrorString) == '')
  {
   $JDTableText   = JD_Number_Year_Table($Y, $JG);

   $TextArea1Text = $JDTableText;
   $TextArea1Rows = 2 + substr_count($TextArea1Text, "\n");
  }





print <<< _HTML

<!doctype html public "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">

<html>

<head>

<meta http-equiv="content-type" content="text/html; charset=UTF-8">

<meta name="robots"    content="noindex,nofollow">
<meta name="googlebot" content="noindex,nofollow">

<title>$_INTERNAL_HTML_TITLE_</title>

<style type='text/css'>
body {color:white; background-color:black;  font-family:Verdana; font-size:15px}

table {font-family:Verdana; font-size:15px; border: 1px solid black;}

td {padding:8px;}

TextArea {color:black; background-color:white; font-family:monospace; font-size:16px; font-weight:bold; border:1px solid black; border-radius:8px; padding:4px; white-space:pre; box-shadow:4px 4px 3px #999999;}

input[type="text"]::-ms-clear {width:0; height:0;}
input[type="text"] {color:black; background-color:white; font-size:18px; font-family:monospace; font-weight:bold; text-align:center; border:2px solid black; border-radius:8px; box-shadow:4px 4px 3px #999999;}
input[type="text"]:focus {background-color:white; font-family:monospace; font-size:18px; border:2px solid red; text-align:center; font-weight:bold; border-radius:8px; box-shadow:4px 4px 3px #999999;}

input[type="submit"]{font-weight:bold;}
input[type="submit"]:hover{box-shadow:2px 2px 5px #222222; font-weight:bold;}

A:link {color:blue; background-color:transparent; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:visited {color:#444444; background-color:transparent; font-family:Verdana; font-size:12px; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

</head>

<body style='color:white; background-color:black;'>

<form name='form1' method='post' action="$_RUN_">
<table width="666" style='font-size:14;'>

<tr><td style='color:white; background-color:#0000DD; text-align:center; border:2px solid white; border-radius:8px 8px 0px 0px; line-height:150%;'>$_PAGE_HEADER_TEXT_</td></tr>

<tr><td style='color:black; background-color:LightCyan; text-align:center; border-radius:0px 0px 8px 9px;'>
<b>Year <input type='text' name='Y' size='6'  maxlength='5' value="$Y" title='Year &nbsp;=&nbsp; &minus;4714 &nbsp;to&nbsp; +9999'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Calendar Mode </b>
<input type='text' name='JG' size='2'  maxlength='1' value="$JG" title='J | G &nbsp;=&nbsp; Julian | Gregorian'>
</td></tr>

<tr><td style='color:silver; background-color:black; text-align:center;'>
<input type='submit' name='ComputeButton' value='C O M P U T E'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<input type='submit' name='InfoButton' value='I N F O'>

</td></tr>


<tr><td style='color:silver; background-color:black; text-align:center;'><br>
Double-Click to Select ALL Text<br><br>
<textarea name='TextArea1' rows="$TextArea1Rows" cols="$TextCols" style='font-size:$TextSize; color:$TxColor; background-color:$BgColor; box-shadow:none; border:2px solid white; padding:8px;' readonly OnDblClick='this.select();' OnMouseUp='return true;'>
$TextArea1Text
</textarea>
</td></tr>


<tr><td style="font-size:12px; color:#888888; background-color:black; text-align:center;">Program by Jay Tanner - $_REVISION_DATE_</td></tr>



</table>
</form>
<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>

</body>

</html>


_HTML;



// *******************************************************
// *******************************************************
// *******************************************************
// *******************************************************

   function Days_in_Month ($Year, $Month, $JGMode='G')
{
   $Y = intval(trim($Year));  if ($Y < -4714 or $Y > 9999) {return FALSE;}
   $m = intval(trim($Month)); if ($m < 1 or $m > 12) {return FALSE;}
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   $mDays = substr("312831303130313130313031", 2*($m-1), 2);
   if (Is_Leap_Year($Y, $JG) and $m == 2) {return $mDays + 1;}
   return $mDays;
}




    function Is_Leap_Year ($Year, $JGMode='G')
{
   $Y = trim($Year);   $Y += ($Y < 0)? 1:0;
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   if ($JG == 'J' and $Y % 4 == 0) {return TRUE;}
   return ($Y % 4 == 0 and !($Y % 100 == 0 and $Y % 400 <> 0));
}




   function JD_Num ($Year, $Month, $Day, $JGMode='G')
{
   $Y = intval(trim($Year));
   $m = intval(trim($Month));
   $d = intval(trim($Day));
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0 : 1;
   $a = floor((14 - $m) / 12);
   $b = (($Y < 0)? $Y+1 : $Y) - $a;
   $c = floor($b/100);
   $J = floor(30.6001*(12*$a + $m + 1))
      + floor(365.25*($b + 4716)) + $d - 1524
      + $M*(floor($c/4) - $c + 2);
   return ("$Y $m $d" <> Inv_JD_Num($J, $JGMode))? FALSE : $J;
}



   function Inv_JD_Num ($JDNumber, $JGMode='G', $RetMode123=1)
{
   $w = intval(trim($RetMode123)); $RMode = ($w < 1 or $w > 3)? 1 : $w;
   $w = trim($JDNumber); if(!is_numeric($w)) {return FALSE;}
   $J = floor($w + 0.5); if($J < 0) {return FALSE;}
   $M = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 0:1;
   $e = $M*floor(($J - 1867216.25)/36524.25);
   $f = $J + $M*($e - floor($e/4) + 1);
   $g = $f + 1524;
   $h = floor(($g - 122.1)/365.25);
   $i = floor(365.25 * $h);
   $j = floor(($g - $i)/30.6001);
   $d = $g - $i - floor(30.6001 * $j);
   $m = $j - 12*floor($j/14) - 1;
   $w = $h + floor((14-$m)/12) - 4716;
   $Y = ($w > 0)? $w : $w-1;

   $w   = '';
   $Mmm = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoW = substr('SunMonTueWedThuFriSat', 3*(($J+1) % 7), 3);
   $Y   = substr('     '.sprintf("%+05d", $Y), -5);

   switch ($RMode)
  {
   case  1: $w = intval(trim($Y))." $m $d"; break;
   case  2: $w = "$Y ".sprintf("%02d", $m).' '.sprintf("%02d", $d); break;
   case  3: $w = "$Y-$Mmm-".sprintf("%02d", $d)."-$DoW"; break;
   default: $w = 'RetMode Switch Error'; break;
  }
   return $w;
}




   function JD_Number_Month_Table ($Year, $Month, $JGMode='G')
{
   $Y = intval(trim($Year));
   $m = intval(trim($Month));
   $JG = (strtoupper(substr(trim($JGMode), 0,1)) == 'J')? 'J':'G';
   $JGStr = ($JG == 'J')? 'Julian':'Gregorian';

   $Mmm = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m-1), 3);

   $mDays = Days_in_Month($Y, $m, $JG);

   $JDNum00 = JD_Num($Y, $m, 1, $JG) - 1;

   $MonthTable = '';
   $dStart     = 1;
   $dEnd       = $mDays;

   $y = sprintf("%d", $Y);
   if($Y > 1000) {$y = str_replace('+', ' ', $y);}

   if ($Y == -4714 and $m == 11 and $JG == 'G')
  {
   return
"
=========================================
 JD NUMBER TABLE  Month Nov / Year -4714
                  Gregorian Calendar
                  Neg|Pos Year = BC|AD

  Calendar Date              Julian Date
 Year Mmm dn DoW  JD Number   JD at 00h
----------------  ---------  -----------
-4714-Nov-24-Mon   0               -0.5
-4714-Nov-25-Tue   1                0.5
-4714-Nov-26-Wed   2                1.5
-4714-Nov-27-Thu   3                2.5
-4714-Nov-28-Fri   4                3.5
-4714-Nov-29-Sat   5                4.5
-4714-Nov-30-Sun   6                5.5";
  }

   for($d=$dStart;   $d <= $dEnd;   $d++)
  {
   $JDNum       = $JDNum00 + $d;
   $JDNumStr    = substr("$JDNum       ", 0,7);
   $JD00h       = $JDNum - 0.5;
   $JD00hStr    = substr("         $JD00h", -9);
   $DateString  = Inv_JD_Num ($JDNum, $JG, 3);
   $MonthTable .= "$DateString   $JDNumStr    $JD00hStr\n";
  }
   if (intval($MonthTable) > 1000) {$MonthTable = str_replace('+', ' ', $MonthTable);}

   $BCMssg = ($Y < 0)? "\n                  Neg|Pos Year = BC|AD":'';

   $MonthTable =
"
=========================================
 JD NUMBER TABLE  Month $Mmm / Year $y
                  $JGStr Calendar$BCMssg

  Calendar Date              Julian Date
 Year Mmm dn DoW  JD Number   JD at 00h
----------------  ---------  -----------
$MonthTable
";
   return RTrim($MonthTable);

} // End of  JD_Number_Month_Table (...)





   function JD_Number_Year_Table ($Year, $JGMode='G')
{
   $Y = intval(trim($Year));
   $y = sprintf("%d", $Y);
   if($Y > 1000) {$y = str_replace('+', ' ', $y);}

   $JG = (strtoupper(substr(trim($JGMode),0,1)) == 'J')? 'J':'G';
   $JGStr = ($JG == 'J')? 'Julian Calendar   ':'Gregorian Calendar';

   $JDNumTableText = '';   $mStart = 1;

   if ($Y == -4714) {$mStart = 11;}

   for ($m = $mStart;   $m <= 12;   $m++)
       {
        $w = JD_Number_Month_Table ($Y, $m, $JG);
        $JDNumTableText .= "$w\n\n";
       }
   $JDNumTableText = RTrim($JDNumTableText);

   $w = "";

   $BCMssg = ($Y < 0)? "\n Neg|Pos Year = BC|AD":'';

// Prepend this text to origin of Gregorian calendar.
   if ($Y == -4714 and $JG == 'G')
  {
   $w =
"=========================================
The mathematical origin of  the Gregorian
calendar is on -4714-Nov-24-Mon,  so this
table does not span a full calendar year.

";
  }

   $JDNumTableText =
"$w=========================================
 JULIAN DAY NUMBER TABLE FOR YEAR: $y$BCMssg

 $JGStr
$JDNumTableText
=========================================";

   return RTrim($JDNumTableText);

}// End of  JD_Number_Year_Table (...)



?>



